@@ -3,7 +3,7 @@ import numpy as np
 
 from enum import Enum
 
-from cctk import File, Ensemble, Molecule, ConformationalEnsemble
+from cctk import File, Ensemble, Molecule, ConformationalEnsemble, OneIndexedArray
 from cctk.helper_functions import get_symbol, compute_distance_between, compute_angle_between, compute_dihedral_between, get_number
 
 import cctk.parse_gaussian as parse
@@ -22,19 +22,20 @@ class JobType(Enum):
     IRC = "irc"
     NMR = "nmr"
     POP = "pop"
+    FORCE = "force"
 
 
 class GaussianFile(File):
     """
-    Class for Gaussian files. Composes ``Ensemble``.
+    Class for Gaussian files. Composes ``ConformationalEnsemble``.
 
     Attributes:
-        molecules (Ensemble): ``Ensemble`` or ``ConformationalEnsemble`` instance
+        molecules (Ensemble): ``ConformationalEnsemble`` instance
         job_types (list): list of `job_type` instances
-        header (str): optional, header of .gjf file
+        route_card (str): optional, route card of .gjf file
+        link0 (dict): optional, dictionary of Link 0 commands (e.g. {"mem": "32GB", "nprocshared": 16})
         footer (str): optional, footer of .gjf file
         success (int): number of successful terminations (should be 1 for an opt, 2 for opt and then freq, 1 for a single point energy, etc)
-        theory (dict): contains information from header
         energies (list): list of energies for each cycle
         scf_iterations (list): number of iterations per cycle
         max_displacements (list): list of max displacement values for each cycle
@@ -46,11 +47,10 @@ class GaussianFile(File):
         gibbs_free_energy (float): gibbs free energy, from vibrational correction
         enthalpy (float): enthalpy, from vibrational correction
         title (str): optional, title of .gjf file
-        conformational_ensemble (Bool): whether or not molecules is an ``Ensemble`` or ``ConformationalEnsemble`` instance
     """
 
     def __init__(
-        self, atomic_numbers, geometries, bonds=None, charges=None, multiplicities=None, job_types=None, theory=None, header=None, footer=None, title="title", conformational_ensemble=False,
+        self, atomic_numbers, geometries, bonds=None, charge=None, multiplicity=None, job_types=None, route_card=None, link0=None, footer=None, title="title",
     ):
         """
         Create new GaussianFile object.
@@ -62,14 +62,17 @@ class GaussianFile(File):
             charges (int): the charge of the molecules
             multiplicities (int): the spin states of the molecules (1 corresponds to singlet, 2 to doublet, 3 to triplet, etc. -- so a multiplicity of 1 is equivalent to S=0)
             job_types (list): list of ``job_type`` instances
-            header (str): optional, header of ``.gjf`` file
+            route_card (str): optional, route card of ``.gjf`` file
+            link0 (dict): optional, Link 0 commands of ``.gjf`` file
             footer (str): optional, footer of ``.gjf`` file
-            theory (dict): optional, contains information from header
             title (str): optional, title of ``.gjf`` file
 		"""
 
-        if header and not isinstance(header, str):
-            raise TypeError("header needs to be a string")
+        if route_card and not isinstance(route_card, str):
+            raise TypeError("route card needs to be a string")
+
+        if link0 and not isinstance(link0, dict):
+            raise TypeError("link0 needs to be a dict")
 
         if footer and not isinstance(footer, str):
             raise TypeError("footer needs to be a string")
@@ -81,65 +84,54 @@ class GaussianFile(File):
             if not all(isinstance(job, JobType) for job in job_types):
                 raise TypeError(f"invalid job type {job}")
 
-        if charges is None:
-            charges = list(np.zeros(shape=len(geometries)))
-
-        if multiplicities is None:
-            multiplicities = list(np.ones(shape=len(geometries)))
-
         if (len(atomic_numbers) > 0) and (len(geometries) > 0):
-            arguments = {"atomic_numbers": atomic_numbers, "geometries": geometries, "bonds": bonds, "charges": charges, "multiplicities": multiplicities}
-            if conformational_ensemble:
-                self.molecules = ConformationalEnsemble(**arguments)
-            else:
-                try:
-                    self.molecules = ConformationalEnsemble(**arguments)
-                    conformational_ensemble = True
-                except AssertionError as e:
-                    self.molecules = Ensemble(**arguments)
+            arguments = {"atomic_numbers": atomic_numbers, "geometries": geometries, "bonds": bonds, "charge": charge, "multiplicity": multiplicity}
+            self.molecules = ConformationalEnsemble(**arguments)
 
-        self.header = header
+        self.route_card = route_card
+        self.link0 = link0
         self.footer = footer
         self.title = title
         self.job_types = job_types
 
     @classmethod
-    def write_molecule_to_file(cls, filename, molecule, header, footer=None, memory=32, cores=16, chk_path=None, title="title", append=False):
+    def write_molecule_to_file(cls, filename, molecule, route_card, link0={"mem": "32GB", "nprocshared": 16}, footer=None, title="title", append=False, print_symbol=False):
         """
         Write a ``.gjf`` file using the given molecule.
 
         Args:
             filename (str): path to the new file
-            molecule (Molecule): which molecule to use -- a``Molecule`` object.
-            memory (int): how many GB of memory to request
-            header (str): header for new file
+            molecule (Molecule): which molecule to use -- a ``Molecule`` object.
+            route_card (str): route card for new file
+            link0 (dict): dictionary of Link 0 commands
             footer (str): footer for new file
-            cores (int): how many CPU cores to request
-            chk_path (str): path to checkpoint file, if desired
             title (str): title of the file, defaults to "title"
             append (Bool): whether or not to append to file using Link1 specifications
+            print_symbol (Bool): whether to print atomic symbols (instead of atomic numbers)
         """
         if not isinstance(molecule, Molecule):
             raise TypeError("need a valid molecule to write a file!")
 
-        if (header is None) or (not isinstance(header, str)):
-            raise ValueError("can't write a file without a header")
+        if (route_card is None) or (not isinstance(route_card, str)):
+            raise ValueError("can't write a file without a route card")
 
         #### generate the text
         text = ""
         if append:
             text += "--Link1--\n"
-        text += f"%nprocshared={int(cores)}\n"
-        text += f"%mem={int(memory)}GB\n"
-        if chk_path:
-            text += f"%chk={chk_path}\n"
 
-        text += f"{header.strip()}\n\n{title}\n\n"
+        if isinstance(link0, dict):
+            for key, val in link0.items():
+                text += f"%{key}={val}\n"
+
+        text += f"{route_card.strip()}\n\n{title}\n\n"
 
         text += f"{int(molecule.charge)} {int(molecule.multiplicity)}\n"
         for index, Z in enumerate(molecule.atomic_numbers, start=1):
             line = molecule.get_vector(index)
-            text += f"{Z:2d} {line[0]:.8f} {line[1]:.8f} {line[2]:.8f}\n"
+            if print_symbol:
+                Z = get_symbol(Z)
+            text += f"{Z:2d}       {line[0]:>13.8f} {line[1]:>13.8f} {line[2]:>13.8f}\n"
 
         text += "\n"
         if footer is not None:
@@ -151,31 +143,32 @@ class GaussianFile(File):
         else:
             super().write_file(filename, text)
 
-    def write_file(self, filename, molecule=None, header=None, footer=None, **kwargs):
+    def write_file(self, filename, molecule=None, route_card=None, link0=None, footer=None, **kwargs):
         """
-        Write a ``.gjf`` file, using object attributes. If no header is specified, the object's header/footer will be used.
+        Write a ``.gjf`` file, using object attributes. If no header/footer is specified, the object's header/footer will be used.
 
         Args:
             filename (str): path to the new file
-            memory (int): how many GB of memory to request
-            cores (int): how many CPU cores to request
-            chk_path (str): path to checkpoint file, if desired
             molecule (int): which molecule to use -- passed to ``self.get_molecule()``.
                 Default is -1 (e.g. the last molecule), but positive integers will select from self.molecules (1-indexed).
                 A ``Molecule`` object can also be passed, in which case that molecule will be written to the file.
-            header (str): header for new file
+            route_card (str): route card for new file
+            link0 (dict): dictionary of Link 0 commands (e.g. {"mem": "32GB", "nprocshared": 16}
             footer (str): footer for new file
         """
         if not isinstance(molecule, Molecule):
             molecule = self.get_molecule(molecule)
 
-        if header is None:
-            header = self.header
+        if route_card is None:
+            route_card = self.route_card
+
+        if link0 is None:
+            link0 = self.link0
 
         if footer is None:
             footer = self.footer
 
-        self.write_molecule_to_file(filename, molecule, header, footer, **kwargs)
+        self.write_molecule_to_file(filename, molecule, route_card, link0, footer, **kwargs)
 
     def num_imaginaries(self):
         """
@@ -204,92 +197,98 @@ class GaussianFile(File):
             filename (str): path to the out file
             return_lines (Bool): whether the lines of the file should be returned
         Returns:
-            GaussianFile object
-            (optional) the lines of the file
+            ``GaussianFile`` object (or list of ``GaussianFile`` objects for Link1 files)
+            (optional) the lines of the file (or list of lines of file for Link1 files)
         """
         if re.search("gjf$", filename):
             return cls._read_gjf_file(filename, return_lines)
 
-        lines = super().read_file(filename)
-        header = parse.search_for_block(lines, "#p", "----")
+        link1_lines = parse.split_link1(super().read_file(filename))
+        files = []
+        for lines in link1_lines:
+            #### automatically assign job types based on header
+            header = parse.search_for_block(lines, "#p", "----")
+            job_types = cls._assign_job_types(header)
 
-        #### automatically assign job types based on header
-        job_types = []
-        for name, member in JobType.__members__.items():
-            if re.search(f" {member.value}", str(header)):
-                job_types.append(member)
+            link0 = parse.extract_link0(lines)
 
-        #### extract parameters
-        success = 0
-        for line in lines:
-            if line.strip().startswith("Normal termination"):
-                success += 1
+            #### extract parameters
+            success = 0
+            for line in lines:
+                if line.strip().startswith("Normal termination"):
+                    success += 1
 
-        (geometries, atom_lists, energies, scf_iterations,) = parse.read_geometries_and_energies(lines)
-        geometries = np.array(geometries)
-        atomic_numbers = []
+            (geometries, atom_list, energies, scf_iterations,) = parse.read_geometries_and_energies(lines)
+            atomic_numbers = []
 
-        #### convert to right datatype
-        for atom_list in atom_lists:
+            #### convert to right datatype
             try:
-                atomic_numbers.append(np.array(atom_list, dtype=np.int8))
+                atomic_numbers = np.array(atom_list, dtype=np.int8)
             except:
-                atomic_numbers.append(np.array(list(map(get_number, atom_list)), dtype=np.int8))
+                atomic_numbers = np.array(list(map(get_number, atom_list)), dtype=np.int8)
 
-        footer = None
-        if re.search("modredundant", str(header)):
-            footer = parse.search_for_block(lines, "^ The following ModRedundant input section", "^ $", count=1, join="\n")
-            footer = "\n".join(list(footer.split("\n"))[1:])  # get rid of the first line
-            footer = "\n".join([" ".join(list(filter(None, line.split(" ")))) for line in footer.split("\n")])
+            footer = None
+            if re.search("modredundant", str(header)):
+                footer = parse.search_for_block(lines, "^ The following ModRedundant input section", "^ $", count=1, join="\n")
+                footer = "\n".join(list(footer.split("\n"))[1:])  # get rid of the first line
+                footer = "\n".join([" ".join(list(filter(None, line.split(" ")))) for line in footer.split("\n")])
 
-        bonds = parse.read_bonds(lines)
-        charge = [int(x) for x in parse.find_parameter(lines, "Multiplicity", expected_length=6, which_field=2)]
-        multip = [int(x) for x in parse.find_parameter(lines, "Multiplicity", expected_length=6, which_field=5)]
+            bonds = parse.read_bonds(lines)
+            charge = parse.find_parameter(lines, "Multiplicity", expected_length=6, which_field=2)[0]
+            multip = parse.find_parameter(lines, "Multiplicity", expected_length=6, which_field=5)[0]
 
-        #### parameters don't get printed every time for opt jobs
-        if JobType.OPT in job_types:
-            bonds = np.repeat(np.array([bonds[0]]), len(atomic_numbers)-1, axis=0).tolist() + [bonds[-1]]
-            charge = list(np.tile(np.array(charge[0]), len(atomic_numbers)-1)) + [charge[-1]]
-            multip = list(np.tile(np.array(multip[0]), len(atomic_numbers)-1)) + [multip[-1]]
+            f = GaussianFile(atomic_numbers, geometries, bonds, job_types=job_types, charge=charge, multiplicity=multip)
+            f.energies = energies
+            f.scf_iterations = scf_iterations
+            f.route_card = header
+            f.link0 = link0
+            f.footer = footer
+            f.success = success
 
-        f = GaussianFile(atomic_numbers, geometries, bonds, job_types=job_types, charges=charge, multiplicities=multip)
-        f.energies = energies
-        f.scf_iterations = scf_iterations
-        f.header = header
-        f.footer = footer
-        f.success = success
+            #### now for some job-type specific attributes
+            if JobType.OPT in job_types:
+                f.rms_forces = parse.find_parameter(lines, "RMS\s+Force", expected_length=5, which_field=2)
+                f.rms_displacements = parse.find_parameter(lines, "RMS\s+Displacement", expected_length=5, which_field=2)
 
-        #### now for some job-type specific attributes
-        if JobType.OPT in job_types:
-            f.rms_forces = parse.find_parameter(lines, "RMS\s+Force", expected_length=5, which_field=2)
-            f.rms_displacements = parse.find_parameter(lines, "RMS\s+Displacement", expected_length=5, which_field=2)
+            if JobType.FREQ in job_types:
+                enthalpies = parse.find_parameter(lines, "thermal Enthalpies", expected_length=7, which_field=6)
+                if len(enthalpies) == 1:
+                    f.enthalpy = enthalpies[0]
+                elif len(enthalpies) > 1:
+                    raise ValueError("too many enthalpies found!")
 
-        if JobType.FREQ in job_types:
-            enthalpies = parse.find_parameter(lines, "thermal Enthalpies", expected_length=7, which_field=6)
-            if len(enthalpies) == 1:
-                f.enthalpy = enthalpies[0]
-            elif len(enthalpies) > 1:
-                raise ValueError("too many enthalpies found!")
+                gibbs_vals = parse.find_parameter(lines, "thermal Free Energies", expected_length=8, which_field=7)
+                if len(gibbs_vals) == 1:
+                    f.gibbs_free_energy = gibbs_vals[0]
+                elif len(gibbs_vals) > 1:
+                    raise ValueError("too many gibbs free energies found!")
 
-            gibbs_vals = parse.find_parameter(lines, "thermal Free Energies", expected_length=8, which_field=7)
-            if len(gibbs_vals) == 1:
-                f.gibbs_free_energy = gibbs_vals[0]
-            elif len(gibbs_vals) > 1:
-                raise ValueError("too many gibbs free energies found!")
+                frequencies = []
+                try:
+                    frequencies += parse.find_parameter(lines, "Frequencies", expected_length=5, which_field=2)
+                    frequencies += parse.find_parameter(lines, "Frequencies", expected_length=5, which_field=3)
+                    frequencies += parse.find_parameter(lines, "Frequencies", expected_length=5, which_field=4)
+                    f.frequencies = sorted(frequencies)
+                except:
+                    raise ValueError("error finding frequencies")
 
-            frequencies = []
-            try:
-                frequencies += parse.find_parameter(lines, "Frequencies", expected_length=5, which_field=2)
-                frequencies += parse.find_parameter(lines, "Frequencies", expected_length=5, which_field=3)
-                frequencies += parse.find_parameter(lines, "Frequencies", expected_length=5, which_field=4)
-                f.frequencies = sorted(frequencies)
-            except:
-                raise ValueError("error finding frequencies")
+            if JobType.NMR in job_types:
+                assert len(f.molecules) == 1, "NMR jobs should not be combined with optimizations!"
+                nmr_shifts = parse.read_nmr_shifts(lines, f.molecules[-1].num_atoms())
+                f.molecules[0].nmr_isotropic = nmr_shifts.view(OneIndexedArray)
+
+            files.append(f)
 
         if return_lines:
-            return f, lines
+            if len(link1_lines) == 1:
+                return files[0], link1_lines[0]
+            else:
+                return files, link1_lines
         else:
-            return f
+            if len(link1_lines) == 1:
+                return files[0]
+            else:
+                return files
 
     @classmethod
     def _read_gjf_file(cls, filename, return_lines=False):
@@ -305,6 +304,7 @@ class GaussianFile(File):
         """
         lines = super().read_file(filename)
         header = None
+        link0 = {}
         footer = None
         header_done = False
         title = None
@@ -316,6 +316,10 @@ class GaussianFile(File):
 
         for idx, line in enumerate(lines):
             if header is None:
+                if re.match("\%", line):
+                    pieces = line[1:].split("=")
+                    link0[pieces[0]] = pieces[1]
+                    continue
                 if re.match("#", line):
                     header = line
                     continue
@@ -364,11 +368,14 @@ class GaussianFile(File):
             atomic_numbers = np.array(list(map(get_number, atomic_numbers)), dtype=np.int8)
 
         geometries = np.array([geometries])
+        job_types = cls._assign_job_types(header)
 
-        f = GaussianFile([atomic_numbers], geometries, charges=[charge], multiplicities=[multip])
-        f.header = header
+        f = GaussianFile(atomic_numbers, geometries, job_types=job_types, charge=charge, multiplicity=multip)
+        f.route_card = header
+        f.link0 = link0
         f.footer = footer
         f.title = title
+        f.success = 0
 
         if return_lines:
             return f, lines
@@ -379,7 +386,7 @@ class GaussianFile(File):
         """
         Returns the last molecule (from an optimization job) or the only molecule (from other jobs).
 
-        If ``num`` is specified, returns that job (1-indexed for positive numbers). So ``job.get_molecule(3)`` will return the 3rd element of ``job.molecules``, not the 4th.
+        If ``num`` is specified, returns ``self.molecules[num]``
         """
         # some methods pass num=None, which overrides setting the default above
         if num is None:
@@ -388,14 +395,10 @@ class GaussianFile(File):
         if not isinstance(num, int):
             raise TypeError("num must be int")
 
-        #### enforce 1-indexing for positive numbers
-        if num > 0:
-            num += -1
-
-        return self.molecules.molecules[num]
+        return self.molecules[num]
 
     @classmethod
-    def write_ensemble_to_file(cls, filename, ensemble, headers, kwargs):
+    def write_ensemble_to_file(cls, filename, ensemble, route_cards, kwargs):
         """
         Writes an Ensemble to a file using Link1 specification.
 
@@ -407,7 +410,26 @@ class GaussianFile(File):
         """
         for idx, molecule in enumerate(ensemble.molecules):
             if idx == 0:
-                cls.write_molecule_to_file(filename, molecule, headers[idx], append=False, **kwargs[idx])
+                cls.write_molecule_to_file(filename, molecule, route_cards[idx], append=False, **kwargs[idx])
             else:
-                cls.write_molecule_to_file(filename, molecule, headers[idx], append=True, **kwargs[idx])
+                cls.write_molecule_to_file(filename, molecule, route_cards[idx], append=True, **kwargs[idx])
 
+
+    @classmethod
+    def _assign_job_types(cls, header):
+        """
+        Assigns ``JobType`` objects from route card.
+
+        For instance, "#p opt freq=noraman" would give an output of [JobType.OPT, JobType.FREQ].
+
+        Args:
+            header (str): Gaussian route card
+
+        Returns:
+            list of ``JobType`` objects
+        """
+        job_types = []
+        for name, member in JobType.__members__.items():
+            if re.search(f" {member.value}", str(header), re.IGNORECASE):
+                job_types.append(member)
+        return job_types
