@@ -33,6 +33,7 @@ class Molecule:
         bonds (nx.Graph): Graph object containing connectivity information (1-indexed)
         charge (int): the charge of the molecule
         multiplicity (int): the spin state of the molecule (1 corresponds to singlet, 2 to doublet, 3 to triplet, etc. -- so a multiplicity of 1 is equivalent to S=0)
+        nmr_isotropic (cctk.OneIndexedArray): NMR isotropic shifts
     """
 
     def __init__(self, atomic_numbers, geometry, name=None, bonds=None, charge=0, multiplicity=1):
@@ -54,7 +55,7 @@ class Molecule:
             geometry = np.array(geometry)
             geometry = geometry.astype(float)
         except:
-            raise TypeError("geometry cannot be cast to ndarray of floats!")
+            raise TypeError("geometry cannot be cast to ``np.ndarray`` of floats!")
 
         if not all(all(isinstance(y, float) for y in x) for x in geometry):
             raise TypeError("each element of self.geometry must be a 3-tuple")
@@ -81,8 +82,8 @@ class Molecule:
                 raise TypeError("multiplicity must be positive integer or castable to positive integer")
         assert multiplicity > 0, "multiplicity must be positive"
 
-        self.atomic_numbers = OneIndexedArray(atomic_numbers, dtype=np.int8)
-        self.geometry = OneIndexedArray(geometry)
+        self.atomic_numbers = atomic_numbers.view(OneIndexedArray)
+        self.geometry = geometry.view(OneIndexedArray)
 
         if bonds:
             for bond in bonds:
@@ -630,9 +631,9 @@ class Molecule:
         The default values should work nicely for low-res MS applications.
 
         Args:
-            formula (np.array, dtype=np.int8): vector containing atoms left to incorporate
-            masses (np.array): list of mass fragments at current iteration
-            weights (np.array): relative weights at current iteration
+            formula (np.ndarray, dtype=np.int8): vector containing atoms left to incorporate
+            masses (np.ndarray): list of mass fragments at current iteration
+            weights (np.ndarray): relative weights at current iteration
             cutoff (float): cutoff for similarity (masses within ``cutoff`` will be combined)
             mass_precision (int): number of decimal places to store for mass
             weight_precision (int): number of decimal places to store for weight
@@ -758,8 +759,8 @@ class Molecule:
 
         try:
             self.bonds.remove_node(number)
-            self.geometry = np.delete(self.geometry, number, axis=0).view(OneIndexedArray)
-            self.atomic_numbers = np.delete(self.atomic_numbers, number).view(OneIndexedArray)
+            self.geometry = np.delete(self.geometry, number - 1, axis=0).view(OneIndexedArray)
+            self.atomic_numbers = np.delete(self.atomic_numbers, number - 1).view(OneIndexedArray)
             return self
         except:
             raise ValueError("removing atom {number} failed!")
@@ -1089,3 +1090,27 @@ class Molecule:
         self.translate_molecule(-self.geometry[atoms].mean(axis=0))
         return self
 
+    @classmethod
+    def combine_molecules(cls, molecule1, molecule2):
+        """
+        Combine two molecules into one final molecule.
+
+        Bonding information is not currently preserved.
+
+        Args:
+            molecule1 (Molecule): 1st molecule
+            molecule2 (Molecule): 2nd molecule
+
+        Returns:
+            new ``Molecule`` object
+        """
+
+        atoms = np.hstack((molecule1.atomic_numbers.T, molecule2.atomic_numbers.T)).view(OneIndexedArray)
+        geoms = np.vstack((molecule1.geometry, molecule2.geometry)).view(OneIndexedArray)
+        charge = molecule1.charge + molecule2.charge
+
+        s1 = (molecule1.multiplicity - 1) / 2
+        s2 = (molecule2.multiplicity - 1) / 2
+        multiplicity = (s1+s2) * 2 + 1
+
+        return Molecule(atoms, geoms, charge=charge, multiplicity=multiplicity)
